/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSClassDescription;

import dstep.foundation.NSArray;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSException;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSClassDescription_bindings;

NSString NSClassDescriptionNeededForClassNotification;

static this ()
{
	NSClassDescriptionNeededForClassNotification = new NSString(bindings.NSClassDescriptionNeededForClassNotification);
}

class NSClassDescription : NSObject
{
	mixin ObjcWrap;

	static void registerClassDescription (NSClassDescription description, Class aClass)
	{
		return invokeObjcSelfClass!(void, "registerClassDescription:forClass:", NSClassDescription, Class)(description, aClass);
	}

	static void invalidateClassDescriptionCache ()
	{
		return invokeObjcSelfClass!(void, "invalidateClassDescriptionCache");
	}

	static NSClassDescription classDescriptionForClass (Class aClass)
	{
		return invokeObjcSelfClass!(NSClassDescription, "classDescriptionForClass:", Class)(aClassreturn result is this.objcObject ? this : (result !is null ? new NSClassDescription(result) : null);	}

	NSArray attributeKeys ()
	{
		return invokeObjcSelf!(NSArray, "attributeKeys");
	}

	NSArray toOneRelationshipKeys ()
	{
		return invokeObjcSelf!(NSArray, "toOneRelationshipKeys");
	}

	NSArray toManyRelationshipKeys ()
	{
		return invokeObjcSelf!(NSArray, "toManyRelationshipKeys");
	}

	NSString inverseForRelationshipKey (NSString relationshipKey)
	{
		return invokeObjcSelf!(NSString, "inverseForRelationshipKey:", NSString)(relationshipKey);
	}
}

template TNSClassDescriptionPrimitives ()
{
	NSClassDescription classDescription ();
	NSArray attributeKeys ();
	NSArray toOneRelationshipKeys ();
	NSArray toManyRelationshipKeys ();
	NSString inverseForRelationshipKey (NSString relationshipKey);
}

