/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSDictionary;

import dstep.foundation.NSArray;
import dstep.foundation.NSEnumerator;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

class NSMutableDictionary : NSDictionary
{
	mixin ObjcWrap;
	mixin TNSMutableDictionaryCreation;
	mixin TNSExtendedMutableDictionary;

	void removeObjectForKey (Object aKey)
	{
		return invokeObjcSelf!(void, "removeObjectForKey:", Object)(aKey);
	}

	void setObject (Object anObject, Object aKey)
	{
		return invokeObjcSelf!(void, "setObject:forKey:", Object, Object)(anObject, aKey);
	}
}

class NSDictionary : NSObject, INSCopying, INSMutableCopying, INSCoding, INSFastEnumeration
{
	mixin ObjcWrap;
	mixin TNSDictionaryCreation;
	mixin TNSFileAttributes;
	mixin TNSExtendedDictionary;

	NSUInteger count ()
	{
		return invokeObjcSelf!(NSUInteger, "count");
	}

	Object objectForKey (Object aKey)
	{
		return invokeObjcSelf!(Object, "objectForKey:", Object)(aKey);
	}

	NSEnumerator keyEnumerator ()
	{
		return invokeObjcSelf!(NSEnumerator, "keyEnumerator");
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	Object mutableCopyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "mutableCopyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithCoder:", NSCoder)(objcObject, aDecoder);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	NSUInteger countByEnumeratingWithState (NSFastEnumerationState* state, id* stackbuf, NSUInteger len)
	{
		return invokeObjcSelf!(NSUInteger, "countByEnumeratingWithState:objects:count:", NSFastEnumerationState*, id*, NSUInteger)(state, stackbuf, len);
	}
}

template TNSDictionaryCreation ()
{
	static Object dictionary ()
	{
		return invokeObjcSelfClass!(Object, "dictionary");
	}

	static Object dictionaryWithObject (Object object, Object key)
	{
		return invokeObjcSelfClass!(Object, "dictionaryWithObject:forKey:", Object, Object)(object, key);
	}

	static Object dictionaryWithObjects (id* objects, id* keys, NSUInteger cnt)
	{
		return invokeObjcSelfClass!(Object, "dictionaryWithObjects:forKeys:count:", id*, id*, NSUInteger)(objects, keys, cnt);
	}

	static Object dictionaryWithObjectsAndKeys (Object dictionaryWithObjectsAndKeys, ...)
	{
		return invokeObjcSelfClass!(Object, "dictionaryWithObjectsAndKeys:", Object)(dictionaryWithObjectsAndKeys);
	}

	static Object dictionaryWithDictionary (NSDictionary dict)
	{
		return invokeObjcSelfClass!(Object, "dictionaryWithDictionary:", NSDictionary)(dict);
	}

	static Object dictionaryWithObjects (NSArray objects, NSArray keys)
	{
		return invokeObjcSelfClass!(Object, "dictionaryWithObjects:forKeys:", NSArray, NSArray)(objects, keys);
	}

	Object initWithObjects (id* objects, id* keys, NSUInteger cnt)
	{
		return invokeObjcSelf!(Object, "initWithObjects:forKeys:count:", id*, id*, NSUInteger)(objects, keys, cnt);
	}

	this (id* objects, id* keys, NSUInteger cnt)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithObjects:forKeys:count:", id*, id*, NSUInteger)(objcObject, objects, keys, cnt);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithObjectsAndKeys (Object initWithObjectsAndKeys, ...)
	{
		return invokeObjcSelf!(Object, "initWithObjectsAndKeys:", Object)(initWithObjectsAndKeys);
	}

	this (Object initWithObjectsAndKeys, ...)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithObjectsAndKeys:", Object)(objcObject, initWithObjectsAndKeys);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithDictionary (NSDictionary otherDictionary)
	{
		return invokeObjcSelf!(Object, "initWithDictionary:", NSDictionary)(otherDictionary);
	}

	this (NSDictionary otherDictionary)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithDictionary:", NSDictionary)(objcObject, otherDictionary);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithDictionary (NSDictionary otherDictionary, bool flag)
	{
		return invokeObjcSelf!(Object, "initWithDictionary:copyItems:", NSDictionary, bool)(otherDictionary, flag);
	}

	this (NSDictionary otherDictionary, bool flag)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithDictionary:copyItems:", NSDictionary, bool)(objcObject, otherDictionary, flag);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithObjects (NSArray objects, NSArray keys)
	{
		return invokeObjcSelf!(Object, "initWithObjects:forKeys:", NSArray, NSArray)(objects, keys);
	}

	this (NSArray objects, NSArray keys)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithObjects:forKeys:", NSArray, NSArray)(objcObject, objects, keys);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	static Object dictionaryWithContentsOfFile (NSString path)
	{
		return invokeObjcSelfClass!(Object, "dictionaryWithContentsOfFile:", NSString)(path);
	}

	static Object dictionaryWithContentsOfURL (NSURL url)
	{
		return invokeObjcSelfClass!(Object, "dictionaryWithContentsOfURL:", NSURL)(url);
	}

	Object initWithContentsOfFile (NSString path)
	{
		return invokeObjcSelf!(Object, "initWithContentsOfFile:", NSString)(path);
	}

	this (NSString path)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithContentsOfFile:", NSString)(objcObject, path);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithContentsOfURL (NSURL url)
	{
		return invokeObjcSelf!(Object, "initWithContentsOfURL:", NSURL)(url);
	}

	this (NSURL url)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithContentsOfURL:", NSURL)(objcObject, url);

		if (result)
			objcObject = ret;

		dObject = this;
	}
}

template TNSMutableDictionaryCreation ()
{
	static Object dictionaryWithCapacity (NSUInteger numItems)
	{
		return invokeObjcSelfClass!(Object, "dictionaryWithCapacity:", NSUInteger)(numItems);
	}

	Object initWithCapacity (NSUInteger numItems)
	{
		return invokeObjcSelf!(Object, "initWithCapacity:", NSUInteger)(numItems);
	}

	this (NSUInteger numItems)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithCapacity:", NSUInteger)(objcObject, numItems);

		if (result)
			objcObject = ret;

		dObject = this;
	}
}

template TNSExtendedMutableDictionary ()
{
	void addEntriesFromDictionary (NSDictionary otherDictionary)
	{
		return invokeObjcSelf!(void, "addEntriesFromDictionary:", NSDictionary)(otherDictionary);
	}

	void removeAllObjects ()
	{
		return invokeObjcSelf!(void, "removeAllObjects");
	}

	void removeObjectsForKeys (NSArray keyArray)
	{
		return invokeObjcSelf!(void, "removeObjectsForKeys:", NSArray)(keyArray);
	}

	void setDictionary (NSDictionary otherDictionary)
	{
		return invokeObjcSelf!(void, "setDictionary:", NSDictionary)(otherDictionary);
	}
}

template TNSExtendedDictionary ()
{
	NSArray allKeys ()
	{
		return invokeObjcSelf!(NSArray, "allKeys");
	}

	NSArray allKeysForObject (Object anObject)
	{
		return invokeObjcSelf!(NSArray, "allKeysForObject:", Object)(anObject);
	}

	NSArray allValues ()
	{
		return invokeObjcSelf!(NSArray, "allValues");
	}

	NSString description ()
	{
		return invokeObjcSelf!(NSString, "description");
	}

	NSString descriptionInStringsFileFormat ()
	{
		return invokeObjcSelf!(NSString, "descriptionInStringsFileFormat");
	}

	NSString descriptionWithLocale (Object locale)
	{
		return invokeObjcSelf!(NSString, "descriptionWithLocale:", Object)(locale);
	}

	NSString descriptionWithLocale (Object locale, NSUInteger level)
	{
		return invokeObjcSelf!(NSString, "descriptionWithLocale:indent:", Object, NSUInteger)(locale, level);
	}

	bool isEqualToDictionary (NSDictionary otherDictionary)
	{
		return invokeObjcSelf!(bool, "isEqualToDictionary:", NSDictionary)(otherDictionary);
	}

	NSEnumerator objectEnumerator ()
	{
		return invokeObjcSelf!(NSEnumerator, "objectEnumerator");
	}

	NSArray objectsForKeys (NSArray keys, Object marker)
	{
		return invokeObjcSelf!(NSArray, "objectsForKeys:notFoundMarker:", NSArray, Object)(keys, marker);
	}

	bool writeToFile (NSString path, bool useAuxiliaryFile)
	{
		return invokeObjcSelf!(bool, "writeToFile:atomically:", NSString, bool)(path, useAuxiliaryFile);
	}

	bool writeToURL (NSURL url, bool atomically)
	{
		return invokeObjcSelf!(bool, "writeToURL:atomically:", NSURL, bool)(url, atomically);
	}

	NSArray keysSortedByValueUsingSelector (SEL comparator)
	{
		return invokeObjcSelf!(NSArray, "keysSortedByValueUsingSelector:", SEL)(comparator);
	}

	void getObjects (id* objects, id* keys)
	{
		return invokeObjcSelf!(void, "getObjects:andKeys:", id*, id*)(objects, keys);
	}
}

