/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSIndexPath;

import dstep.foundation.NSObject;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

class NSIndexPath : NSObject, INSCopying, INSCoding
{
	mixin ObjcWrap;

	static Object indexPathWithIndex (NSUInteger index)
	{
		return invokeObjcSelfClass!(Object, "indexPathWithIndex:", NSUInteger)(index);
	}

	static Object indexPathWithIndexes (NSUInteger* indexes, NSUInteger length)
	{
		return invokeObjcSelfClass!(Object, "indexPathWithIndexes:length:", NSUInteger*, NSUInteger)(indexes, length);
	}

	Object initWithIndex (NSUInteger index)
	{
		return invokeObjcSelf!(Object, "initWithIndex:", NSUInteger)(index);
	}

	this (NSUInteger index)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithIndex:", NSUInteger)(objcObject, index);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithIndexes (NSUInteger* indexes, NSUInteger length)
	{
		return invokeObjcSelf!(Object, "initWithIndexes:length:", NSUInteger*, NSUInteger)(indexes, length);
	}

	this (NSUInteger* indexes, NSUInteger length)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithIndexes:length:", NSUInteger*, NSUInteger)(objcObject, indexes, length);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	NSIndexPath indexPathByAddingIndex (NSUInteger index)
	{
		id result = invokeObjcSelf!(id, "indexPathByAddingIndex:", NSUInteger)(indexreturn result is this.objcObject ? this : (result !is null ? new NSIndexPath(result) : null);	}

	NSIndexPath indexPathByRemovingLastIndex ()
	{
		id result = invokeObjcSelf!(id, "indexPathByRemovingLastIndex"return result is this.objcObject ? this : (result !is null ? new NSIndexPath(result) : null);	}

	NSUInteger indexAtPosition (NSUInteger position)
	{
		return invokeObjcSelf!(NSUInteger, "indexAtPosition:", NSUInteger)(position);
	}

	NSUInteger length ()
	{
		return invokeObjcSelf!(NSUInteger, "length");
	}

	void getIndexes (NSUInteger* indexes)
	{
		return invokeObjcSelf!(void, "getIndexes:", NSUInteger*)(indexes);
	}

	int compare (NSIndexPath otherObject)
	{
		return invokeObjcSelf!(int, "compare:", NSIndexPath)(otherObject);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithCoder:", NSCoder)(objcObject, aDecoder);

		if (result)
			objcObject = ret;

		dObject = this;
	}
}

