/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSNetServices;

import dstep.foundation.NSArray;
import dstep.foundation.NSData;
import dstep.foundation.NSDate;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSInputStream;
import dstep.foundation.NSObject;
import dstep.foundation.NSOutputStream;
import dstep.foundation.NSRunLoop;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSNetServices_bindings;

alias NSInteger NSNetServicesError;
alias NSUInteger NSNetServiceOptions;

const NSString NSNetServicesErrorCode;
const NSString NSNetServicesErrorDomain;

enum
{
	NSNetServicesUnknownError = -72000,
	NSNetServicesCollisionError = -72001,
	NSNetServicesNotFoundError = -72002,
	NSNetServicesActivityInProgress = -72003,
	NSNetServicesBadArgumentError = -72004,
	NSNetServicesCancelledError = -72005,
	NSNetServicesInvalidError = -72006,
	NSNetServicesTimeoutError = -72007
}

enum
{
	NSNetServiceNoAutoRename = 1 << 0
}

static this ()
{
	NSNetServicesErrorCode = new NSString(bindings.NSNetServicesErrorCode);
	NSNetServicesErrorDomain = new NSString(bindings.NSNetServicesErrorDomain);
}

class NSNetService : NSObject
{
	mixin ObjcWrap;

	Object initWithDomain (NSString domain, NSString type, NSString name, int port)
	{
		return invokeObjcSelf!(Object, "initWithDomain:type:name:port:", NSString, NSString, NSString, int)(domain, type, name, port);
	}

	this (NSString domain, NSString type, NSString name, int port)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithDomain:type:name:port:", NSString, NSString, NSString, int)(objcObject, domain, type, name, port);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithDomain (NSString domain, NSString type, NSString name)
	{
		return invokeObjcSelf!(Object, "initWithDomain:type:name:", NSString, NSString, NSString)(domain, type, name);
	}

	this (NSString domain, NSString type, NSString name)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithDomain:type:name:", NSString, NSString, NSString)(objcObject, domain, type, name);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	void setDelegate (Object delegate_)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(delegate_);
	}

	void scheduleInRunLoop (NSRunLoop aRunLoop, NSString mode)
	{
		return invokeObjcSelf!(void, "scheduleInRunLoop:forMode:", NSRunLoop, NSString)(aRunLoop, mode);
	}

	void removeFromRunLoop (NSRunLoop aRunLoop, NSString mode)
	{
		return invokeObjcSelf!(void, "removeFromRunLoop:forMode:", NSRunLoop, NSString)(aRunLoop, mode);
	}

	NSString domain ()
	{
		return invokeObjcSelf!(NSString, "domain");
	}

	NSString type ()
	{
		return invokeObjcSelf!(NSString, "type");
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	NSArray addresses ()
	{
		return invokeObjcSelf!(NSArray, "addresses");
	}

	NSInteger port ()
	{
		return invokeObjcSelf!(NSInteger, "port");
	}

	void publish ()
	{
		return invokeObjcSelf!(void, "publish");
	}

	void publishWithOptions (uint options)
	{
		return invokeObjcSelf!(void, "publishWithOptions:", uint)(options);
	}

	void resolve ()
	{
		return invokeObjcSelf!(void, "resolve");
	}

	void stop ()
	{
		return invokeObjcSelf!(void, "stop");
	}

	static NSDictionary dictionaryFromTXTRecordData (NSData txtData)
	{
		return invokeObjcSelfClass!(NSDictionary, "dictionaryFromTXTRecordData:", NSData)(txtData);
	}

	static NSData dataFromTXTRecordDictionary (NSDictionary txtDictionary)
	{
		return invokeObjcSelfClass!(NSData, "dataFromTXTRecordDictionary:", NSDictionary)(txtDictionary);
	}

	NSString hostName ()
	{
		return invokeObjcSelf!(NSString, "hostName");
	}

	void resolveWithTimeout (double timeout)
	{
		return invokeObjcSelf!(void, "resolveWithTimeout:", double)(timeout);
	}

	bool getInputStream (NSInputStream** inputStream, NSOutputStream** outputStream)
	{
		return invokeObjcSelf!(bool, "getInputStream:outputStream:", NSInputStream**, NSOutputStream**)(inputStream, outputStream);
	}

	bool setTXTRecordData (NSData recordData)
	{
		return invokeObjcSelf!(bool, "setTXTRecordData:", NSData)(recordData);
	}

	NSData TXTRecordData ()
	{
		return invokeObjcSelf!(NSData, "TXTRecordData");
	}

	void startMonitoring ()
	{
		return invokeObjcSelf!(void, "startMonitoring");
	}

	void stopMonitoring ()
	{
		return invokeObjcSelf!(void, "stopMonitoring");
	}
}

class NSNetServiceBrowser : NSObject
{
	mixin ObjcWrap;
	mixin TNSDeprecated;

	Object init ()
	{
		return invokeObjcSelf!(Object, "init");
	}

	this ()
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "init")(objcObject);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	void setDelegate (Object delegate_)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(delegate_);
	}

	void scheduleInRunLoop (NSRunLoop aRunLoop, NSString mode)
	{
		return invokeObjcSelf!(void, "scheduleInRunLoop:forMode:", NSRunLoop, NSString)(aRunLoop, mode);
	}

	void removeFromRunLoop (NSRunLoop aRunLoop, NSString mode)
	{
		return invokeObjcSelf!(void, "removeFromRunLoop:forMode:", NSRunLoop, NSString)(aRunLoop, mode);
	}

	void searchForBrowsableDomains ()
	{
		return invokeObjcSelf!(void, "searchForBrowsableDomains");
	}

	void searchForRegistrationDomains ()
	{
		return invokeObjcSelf!(void, "searchForRegistrationDomains");
	}

	void searchForServicesOfType (NSString type, NSString domainString)
	{
		return invokeObjcSelf!(void, "searchForServicesOfType:inDomain:", NSString, NSString)(type, domainString);
	}

	void stop ()
	{
		return invokeObjcSelf!(void, "stop");
	}
}

template TNSDeprecated ()
{
	void searchForAllDomains ()
	{
		return invokeObjcSelf!(void, "searchForAllDomains");
	}
}

template TNSNetServiceDelegateMethods ()
{
	void netServiceWillPublish (NSNetService sender);
	void netServiceDidPublish (NSNetService sender);
	void netService (NSNetService sender, NSDictionary errorDict);
	void netServiceWillResolve (NSNetService sender);
	void netServiceDidResolveAddress (NSNetService sender);
	void netService (NSNetService sender, NSDictionary errorDict);
	void netServiceDidStop (NSNetService sender);
	void netService (NSNetService sender, NSData data);
}

template TNSNetServiceBrowserDelegateMethods ()
{
	void netServiceBrowserWillSearch (NSNetServiceBrowser aNetServiceBrowser);
	void netServiceBrowserDidStopSearch (NSNetServiceBrowser aNetServiceBrowser);
	void netServiceBrowser (NSNetServiceBrowser aNetServiceBrowser, NSDictionary errorDict);
	void netServiceBrowser (NSNetServiceBrowser aNetServiceBrowser, NSString domainString, bool moreComing);
	void netServiceBrowser (NSNetServiceBrowser aNetServiceBrowser, NSNetService aNetService, bool moreComing);
	void netServiceBrowser (NSNetServiceBrowser aNetServiceBrowser, NSString domainString, bool moreComing);
	void netServiceBrowser (NSNetServiceBrowser aNetServiceBrowser, NSNetService aNetService, bool moreComing);
}

