/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSNotificationQueue;

import dstep.foundation.NSArray;
import dstep.foundation.NSNotification;
import dstep.foundation.NSNotificationCenter;
import dstep.foundation.NSObject;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

alias NSUInteger NSPostingStyle;
alias NSUInteger NSNotificationCoalescing;

enum
{
	NSPostWhenIdle = 1,
	NSPostASAP = 2,
	NSPostNow = 3
}

enum
{
	NSNotificationNoCoalescing = 0,
	NSNotificationCoalescingOnName = 1,
	NSNotificationCoalescingOnSender = 2
}

class NSNotificationQueue : NSObject
{
	mixin ObjcWrap;

	static Object defaultQueue ()
	{
		return invokeObjcSelfClass!(Object, "defaultQueue");
	}

	Object initWithNotificationCenter (NSNotificationCenter notificationCenter)
	{
		return invokeObjcSelf!(Object, "initWithNotificationCenter:", NSNotificationCenter)(notificationCenter);
	}

	this (NSNotificationCenter notificationCenter)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithNotificationCenter:", NSNotificationCenter)(objcObject, notificationCenter);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	void enqueueNotification (NSNotification notification, uint postingStyle)
	{
		return invokeObjcSelf!(void, "enqueueNotification:postingStyle:", NSNotification, uint)(notification, postingStyle);
	}

	void enqueueNotification (NSNotification notification, uint postingStyle, NSUInteger coalesceMask, NSArray modes)
	{
		return invokeObjcSelf!(void, "enqueueNotification:postingStyle:coalesceMask:forModes:", NSNotification, uint, NSUInteger, NSArray)(notification, postingStyle, coalesceMask, modes);
	}

	void dequeueNotificationsMatching (NSNotification notification, NSUInteger coalesceMask)
	{
		return invokeObjcSelf!(void, "dequeueNotificationsMatching:coalesceMask:", NSNotification, NSUInteger)(notification, coalesceMask);
	}
}

